from django.contrib import admin

try:
    from django.contrib.admin.util import flatten_fieldsets
except ImportError:
    from django.contrib.admin.utils import flatten_fieldsets

from django.utils.translation import ugettext_lazy as _

from models import TrackingLog


class AdminPathFilter(admin.SimpleListFilter):
    title = _('admin')
    parameter_name = 'in_admin'

    def lookups(self, request, model_admin):
        return (
            ('inside', _('inside admin')),
            ('outside', _('outside admin')),
        )

    def queryset(self, request, queryset):
        if self.value() == 'inside':
            return queryset.filter(path__contains='/admin/')
        if self.value() == 'outside':
            return queryset.exclude(path__contains='/admin/')
        return queryset


class TrackingLogAdmin(admin.ModelAdmin):
    readonly_fields = ("created", 'modified', 'full_path')
    date_hierarchy = 'created'
    list_display = ['pk', 'user', 'request_method', 'status_code', 'ip_address', 'path', 'params_get', 'params_post',
                    'is_secure', 'is_ajax', 'is_debug', 'created']
    list_filter = [AdminPathFilter, 'is_secure', 'is_ajax', 'is_debug', 'created', 'status_code', 'request_method', 'user']
    search_fields = ['session', 'user_agent', 'params_get', 'params_post', 'ip_address', 'path']
    fields = (
        ('created', 'modified'),
        ('request_method', 'path', 'status_code'),
        ('is_secure', 'is_ajax', 'is_debug'),
        ('full_path',),
        ('params_get',),
        ('params_post',),
        ('session',),
        ('user', 'ip_address', 'user_agent'),
        ('language_code', 'timezone',),
        ('content_type',),
        ('response',),
    )

    def has_add_permission(self, request):
        return False

    def get_readonly_fields(self, request, obj=None):
        if self.declared_fieldsets:
            return flatten_fieldsets(self.declared_fieldsets)
        else:
            return list(set(
                [field.name for field in self.opts.local_fields] +
                [field.name for field in self.opts.local_many_to_many]
            ))

admin.site.register(TrackingLog, TrackingLogAdmin)



import json
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseForbidden, HttpResponse
from django.utils.decorators import method_decorator
from django.views.generic import DetailView


from models import TrackingLog


class JsonResponse(HttpResponse):
    def __init__(self, json_obj):
        content = json.dumps(json_obj, indent=4)
        super(JsonResponse, self).__init__(
            content=content, mimetype='application/json; charset=utf8'
        )


class TrackingLogDetailView(DetailView):
    model = TrackingLog

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_active or not request.user.is_superuser:
            return HttpResponseForbidden()
        tracking_log = self.get_object()

        try:
            json_obj = json.loads(tracking_log.response)
            return JsonResponse(json_obj)
        except (TypeError, ValueError):
            return HttpResponse(tracking_log.response)



__author__ = 'Erik Telepovsky'

import sys

from django.db import utils

try:
    import psycopg2 as Database
    import psycopg2.extensions
except ImportError as e:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured("Error loading psycopg2 module: %s" % e)

DatabaseError = Database.DatabaseError

from models import TrackingLog
from settings import DETECTIVE_TRACK_AJAX_REQUESTS, DETECTIVE_TRACK_ANONYMOUS_REQUESTS, DETECTIVE_SAVE_ERROR_RESPONSES, \
    DETECTIVE_DEBUG, DETECTIVE_SAVE_RESPONSES


class TrackingMiddleware(object):
    def process_response(self, request, response):
        # Ajax request
        if request.is_ajax() and not DETECTIVE_TRACK_AJAX_REQUESTS:
            return response

        # Currently logged in user
        try:
            user = request.user if request.user.is_authenticated() else None
        except AttributeError:
            return response

        # Anonymous request
        if user is None and not DETECTIVE_TRACK_ANONYMOUS_REQUESTS:
            return response

        # IP address
        ip = request.META.get('HTTP_X_FORWARDED_FOR', None)
        if ip:
            # X_FORWARDED_FOR returns client1, proxy1, proxy2,...
            ip = ip.split(', ')[0]
        else:
            ip = request.META.get('REMOTE_ADDR', None)

        # Language code
        language_code = getattr(request, 'LANGUAGE_CODE', None)

        # Timezone
        timezone = request.META.get('TZ', None)

        # User agent
        user_agent = request.META.get('HTTP_USER_AGENT', None)

        # Content type
        content_type = request.META.get('CONTENT_TYPE', None)

        # Response content
        response_content = None
        if response:
            if response.status_code == 500 and DETECTIVE_SAVE_ERROR_RESPONSES:
                #response_content = response.content # this is only useful if DEBUG is True
                # if DEBUG is False, simulate that DEBUG is enabled and get technical exception details:
                from django.views import debug
                exc_info = sys.exc_info()
                debug_response = debug.technical_500_response(request, *exc_info)
                response_content = debug_response.content
            elif response.status_code != 500 and DETECTIVE_SAVE_RESPONSES:
                response_content = response.content

        # Store to database
        try:
            TrackingLog.objects.create(
                user=user,
                request_method=request.META['REQUEST_METHOD'],
                #    remote_addr = request.META['REMOTE_ADDR'],
                ip_address=ip,
                path=request.path,
                #    query_string = request.META['QUERY_STRING'],
                #    full_path = request.get_full_path(),
                params_get=str(dict(request.GET)),
                params_post=str(dict(request.POST)),
                session=str(dict(request.session)),
                status_code=response.status_code,
                language_code=language_code,
                timezone=timezone,
                content_type=content_type,
                user_agent=user_agent,
                response=response_content,
                is_secure=request.is_secure(),
                is_ajax=request.is_ajax(),
                is_debug=DETECTIVE_DEBUG
            )
        except utils.DatabaseError:
            pass

        # Return original response
        return response



from django.conf import settings

# Site debug
DETECTIVE_DEBUG = getattr(settings, 'DEBUG', False)

# If True, response data will be saved in database
DETECTIVE_SAVE_RESPONSES = getattr(settings, 'DETECTIVE_SAVE_RESPONSES', False)

# If True, internal server error responses with exception (500) are stored in database
DETECTIVE_SAVE_ERROR_RESPONSES = getattr(settings, 'DETECTIVE_SAVE_ERROR_RESPONSES', True)

# Should Ajax requests be tracked?
DETECTIVE_TRACK_AJAX_REQUESTS = getattr(settings, 'DETECTIVE_TRACK_AJAX_REQUESTS', True)

# Should anonymous requests be tracked?
DETECTIVE_TRACK_ANONYMOUS_REQUESTS = getattr(settings, 'DETECTIVE_TRACK_ANONYMOUS_REQUESTS', True)



from django.conf.urls import patterns, url

from views import TrackingLogDetailView


urlpatterns = patterns('',
    url(r'^trackinglog/detail/(?P<pk>[-\d]+)/$', TrackingLogDetailView.as_view(), name='trackinglog_detail'),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='TrackingLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('request_method', models.CharField(max_length=16, verbose_name='request method')),
                ('ip_address', models.GenericIPAddressField(default=None, null=True, verbose_name='IP address', blank=True)),
                ('path', models.CharField(max_length=1024, verbose_name='path')),
                ('params_get', models.TextField(verbose_name='GET params')),
                ('params_post', models.TextField(verbose_name='POST params')),
                ('session', models.TextField(verbose_name='session')),
                ('status_code', models.SmallIntegerField(verbose_name='status code')),
                ('language_code', models.CharField(default=None, max_length=2, null=True, verbose_name='language code', blank=True)),
                ('timezone', models.CharField(default=None, max_length=128, null=True, verbose_name='timezone', blank=True)),
                ('content_type', models.CharField(default=None, max_length=128, null=True, verbose_name='content type', blank=True)),
                ('user_agent', models.CharField(default=None, max_length=1024, null=True, verbose_name='user agent', blank=True)),
                ('response', models.TextField(default=None, null=True, verbose_name='response', blank=True)),
                ('is_secure', models.BooleanField(verbose_name='secure')),
                ('is_ajax', models.BooleanField(verbose_name='ajax')),
                ('is_debug', models.BooleanField(verbose_name='debug')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='created')),
                ('user', models.ForeignKey(default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, verbose_name='user')),
            ],
            options={
                'ordering': ('-created',),
                'db_table': 'detective_trackinglogs',
                'verbose_name': 'tracking log',
                'verbose_name_plural': 'tracking logs',
            },
        ),
    ]



import ast
import urllib

from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

# Django 1.5 support, falls back to auth.User to transparently work with <1.5
try:
    from django.conf import settings
    User = settings.AUTH_USER_MODEL
except ImportError:
    from django.contrib.auth.models import User


class TrackingLog(models.Model):
    user = models.ForeignKey(User, verbose_name=_(u'user'),
        blank=True, null=True, default=None)
    request_method = models.CharField(max_length=16, verbose_name=_(u'request method'))
#    remote_addr = models.CharField(max_length=255)
    ip_address = models.GenericIPAddressField(verbose_name=_(u'IP address'),
        blank=True, null=True, default=None)
    path = models.CharField(max_length=1024, verbose_name=_(u'path'))
#    query_string = models.CharField(max_length=255)
#    full_path = models.CharField(max_length=255)
    params_get = models.TextField(verbose_name=_(u'GET params'))
    params_post = models.TextField(verbose_name=_(u'POST params'))
    session = models.TextField(verbose_name=_(u'session'))
    status_code = models.SmallIntegerField(verbose_name=_(u'status code'))
    language_code = models.CharField(max_length=2, verbose_name=_(u'language code'),
        blank=True, null=True, default=None)
    timezone = models.CharField(max_length=128, verbose_name=_(u'timezone'),
        blank=True, null=True, default=None)
    content_type = models.CharField(max_length=128, verbose_name=_(u'content type'),
        blank=True, null=True, default=None)
    user_agent = models.CharField(max_length=1024, verbose_name=_(u'user agent'),
        blank=True, null=True, default=None)
    response = models.TextField(verbose_name=_(u'response'),
        blank=True, null=True, default=None)
    is_secure = models.BooleanField(verbose_name=_(u'secure'))
    is_ajax = models.BooleanField(verbose_name=_(u'ajax'))
    is_debug = models.BooleanField(verbose_name=_(u'debug'))
    created = models.DateTimeField(verbose_name=_(u'created'), auto_now_add=True)
    modified = models.DateTimeField(verbose_name=_(u'modified'), auto_now=True)

    class Meta:
        db_table = 'detective_trackinglogs'
        verbose_name = _(u'tracking log')
        verbose_name_plural = _(u'tracking logs')
        ordering = ('-created', )

    def get_absolute_url(self):
        return reverse('detective:trackinglog_detail', args=(self.pk, ))

    @property
    def full_path(self):
        return u'%s?%s' % (self.path, self.query_string) if self.query_string else self.path

    @property
    def query_string(self):
        get_params = ast.literal_eval(self.params_get)
        query_pairs = [(k, v) for k, vlist in get_params.iteritems() for v in vlist]
        return urllib.urlencode(query_pairs)
